@@ -20,6 +20,7 @@ class Nsvh1(sabr.SabrABC):
         array([22.45639334, 14.89800673,  8.88641613,  4.65917923,  2.10575204])
     """
 
+    model_type = "Nsvh"
     beta = 0.0  # beta is already defined in the parent class, but the default value set as 0
     is_atmvol = False
 
@@ -43,7 +44,7 @@ class Nsvh1(sabr.SabrABC):
             intr: interest rate (domestic interest rate)
             divr: dividend/convenience yield (foreign interest rate)
             is_fwd: if True, treat `spot` as forward price. False by default.
-            is_atmvol: If True, use `sigma` as the ATM normal vol
+            is_atmvol: If True, use `sigma` as the ATM normal vol. False by default.
         """
         # Make sure beta = 0
         if beta is not None and not np.isclose(beta, 0.0):
