@@ -49,17 +49,12 @@ class SabrMcCond(sabr.SabrABC, sv.CondMcBsmABC):
         int_var = scint.simps(sigma_paths ** 2, dx=1, axis=0) / n_dt
         vol_cond = rhoc * np.sqrt(int_var)
         if np.isclose(self.beta, 0):
-            fwd_cond = rho_sigma / self.vov * (sigma_final - 1)
+            spot_cond = rho_sigma / self.vov * (sigma_final - 1)
         else:
-            fwd_cond = np.exp(
-                rho_sigma
-                * (
-                    1.0 / self.vov * (sigma_final - 1)
-                    - 0.5 * rho_sigma * int_var * texp
-                )
-            )
+            spot_cond = 1.0 / self.vov * (sigma_final - 1) - 0.5 * rho_sigma * int_var * texp
+            np.exp(rho_sigma * spot_cond, out=spot_cond)
 
-        return fwd_cond, vol_cond, log_rn_deriv
+        return spot_cond, vol_cond, log_rn_deriv
 
     def price(self, strike, spot, texp, cp=1):
         fwd = self.forward(spot, texp)
@@ -122,7 +117,7 @@ class SabrMcExactCai2017(sabr.SabrABC, sv.CondMcBsmABC):
     comb_coef = None
     nn = None
 
-    def set_mc_params(self, n_path=10000, m_inv=20, m_euler=20, n_euler=35, rn_seed=None, antithetic=True):
+    def set_num_params(self, n_path=10000, m_inv=20, m_euler=20, n_euler=35, rn_seed=None, antithetic=True):
         """
         Set MC parameters
 
@@ -147,9 +142,6 @@ class SabrMcExactCai2017(sabr.SabrABC, sv.CondMcBsmABC):
         assert abs(self.comb_coef.sum()-1) < 1e-8
         self.nn = np.arange(0, self.m_euler + self.n_euler + 0.1)
 
-    def vol_paths(self, tobs, mu=0):
-        return None
-
     def sigma_final(self, vovn):
         """
         Final Sigma
@@ -163,10 +155,10 @@ class SabrMcExactCai2017(sabr.SabrABC, sv.CondMcBsmABC):
         """
 
         if self.antithetic:
-            zz = self.rng.normal(size=self.n_path // 2)
+            zz = self.rng.standard_normal(size=self.n_path // 2)
             zz = np.hstack([zz, -zz])
         else:
-            zz = self.rng.normal(size=self.n_path)
+            zz = self.rng.standarad_normal(size=self.n_path)
 
         sigma_T = np.exp(vovn * (zz - vovn/2))
         return sigma_T
